package de.hdm_stuttgart.de.testing.dom;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;

import javax.xml.parsers.ParserConfigurationException;

import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.xml.sax.SAXException;

import de.hdm_stuttgart.mi.sda1.saxhtml.tools.ContentRedirect;
import de.hdm_stuttgart.mi.sda1.saxhtml.v1.Memo2Html;

/**
 * Turning &lt;memo&gt; documents to HTML as being shown in sample document memo.xml.2.sample.html 

 */
@SuppressWarnings("javadoc")
public abstract class ConversionTest {
   
   public final String xmlInputFileName, htmlOutputFileName; 
   final ContentRedirect saxHandler;
   protected String errorInitString = null;

   protected Element htmlRootElement; 

   protected ConversionTest(final String xmlInputFileName, final ContentRedirect saxHandler, final String outputExtension) {
      
      this.saxHandler = saxHandler;
      
      this.xmlInputFileName = xmlInputFileName;
      this.htmlOutputFileName = xmlInputFileName + outputExtension;
      
      final PrintStream out;

      try {
         out = new PrintStream(htmlOutputFileName);
      } catch (FileNotFoundException e1) {
         errorInitString = "Unable to open file '" + htmlOutputFileName + "' for writing";
         return;
      }
      saxHandler.setOutputStream(out);
      final Memo2Html memo2html;
      try {
         memo2html = new Memo2Html(saxHandler);
      } catch (SAXException | ParserConfigurationException e2) {
         e2.printStackTrace();
         return;
      }
      try {
         memo2html.parse(xmlInputFileName);
      } catch (IOException e1) {
         errorInitString = "Unable parse file '" + xmlInputFileName + "'";
         return;
      } catch (SAXException e) {
         e.printStackTrace();
      }
      final SAXBuilder parser = new SAXBuilder();
      
      try {
         htmlRootElement = parser.build(htmlOutputFileName).getRootElement();
      } catch (JDOMException | IOException e1) {
         errorInitString = "Unable to parse file '" + htmlOutputFileName + "', see stack trace for further information";
         e1.printStackTrace();
      }
      out.close();
   }
}