package de.hdm_stuttgart.de.testing.dom;

import java.util.List;

import org.jdom2.Element;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

/**
 * @author goik
 *
 */
public class DomAssert {
   
   
   final static XPathFactory xpf = XPathFactory.instance();

   /**
 * @param msg
 * @param context
 * @param xpath
 * @param expectedNodeCount
 */
public static void assertNumberOfNodes(final String msg, final Element context, final String xpath, int expectedNodeCount) {
      
      @SuppressWarnings({ "unchecked", "rawtypes" })
      final XPathExpression<Element> xpathExpr = (XPathExpression<Element>) (XPathExpression) xpf.compile(xpath);

      final List<Element> matchedNodes = xpathExpr.evaluate(context);
      
      if (expectedNodeCount != matchedNodes.size()) {
         throw new AssertXpathResult(msg + "\nExpected " + expectedNodeCount + " node" + (expectedNodeCount == 1? "":"s") + " for xpath '" + xpath + 
               "' but found " + matchedNodes.size());
      }
   } 
   
   /**
 * @param msg
 * @param context
 * @param xpath
 * @param expectedContent
 */
public static void assertSingleNodeContent(final String msg, final Element context, final String xpath, String expectedContent) {
      
      @SuppressWarnings({ "unchecked", "rawtypes" })
      final XPathExpression<Element> xpathExpr = (XPathExpression<Element>) (XPathExpression) xpf.compile(xpath);

      final List<Element> matchedNodes = xpathExpr.evaluate(context);
      if (1 == matchedNodes.size()) {
         final String content = matchedNodes.get(0).getValue();
         if (!content.equals(expectedContent)) {
            throw new AssertXpathResult(msg + "\nExpected content '" + expectedContent + "' for xpath '" + xpath + "' but found '" + content + "'");
         }
      } else {
         throw new AssertXpathResult("\nExactly one node expected for xpath '" + xpath + "' but found " + matchedNodes.size());
      }
   }

   /**
 * @param msg
 * @param context
 * @param xpath
 * @param expectedTagName
 * @param expectedContent
 */
public static void assertSingleNodeContent(final String msg, final Element context, final String xpath, final String expectedTagName, String expectedContent) {
      
      @SuppressWarnings({ "unchecked", "rawtypes" })
      final XPathExpression<Element> xpathExpr = (XPathExpression<Element>) (XPathExpression) xpf.compile(xpath);

      final List<Element> matchedNodes = xpathExpr.evaluate(context);
      if (1 == matchedNodes.size()) {
         
         final Element element = matchedNodes.get(0);
         
         final String elementTagName = element.getName();
         if (!expectedTagName.equals(elementTagName)) {
            throw new AssertXpathResult(msg + "\nExpected <" + expectedTagName  + "> for xpath '" + xpath + "' but found <" +
                  elementTagName + ">");
         }
         final String content = element.getValue();
         if (!expectedContent.equals(content)) {
            throw new AssertXpathResult(msg + "\nExpected element content '" + expectedContent  + "' for xpath '" + xpath + "' but found '" +
                                       content + "'");
         }

      } else {
         throw new AssertXpathResult("Exactly node expected for xpath '" + xpath + "' but found " + matchedNodes.size());
      }
   }
}
